@@ -5,6 +5,7 @@
  * Author:
  *    Jody Goldberg 2000-2006 (jody@gnome.org)
  *    Miguel de Icaza 1998, 1999 (miguel@kernel.org)
+ *    Copyright (C) 2000-2009 Morten Welinder (terra@gnome.org)
  */
 #include <gnumeric-config.h>
 #include "gnumeric.h"
@@ -24,7 +25,8 @@
 #include "sheet-object-cell-comment.h"
 #include "sheet-style.h"
 #include "parse-util.h"
-#include <goffice/utils/go-glib-extras.h>
+
+#include <goffice/goffice.h>
 
 /**
  * gnm_cell_cleanout :
@@ -56,14 +58,11 @@ gnm_cell_cleanout (GnmCell *cell)
 		cell->base.texpr = NULL;
 	}
 
-	if (cell->value != NULL) {
-		value_release (cell->value);
-		cell->value = NULL;
-	}
-	if (cell->rendered_value != NULL) {
-		gnm_rendered_value_destroy (cell->rendered_value);
-		cell->rendered_value = NULL;
-	}
+	value_release (cell->value);
+	cell->value = NULL;
+
+	gnm_cell_unrender (cell);
+
 	if (cell->row_info != NULL)
 		cell->row_info->needs_respan = TRUE;
 }
@@ -129,8 +128,7 @@ gnm_cell_assign_value (GnmCell *cell, GnmValue *v)
 	g_return_if_fail (cell);
 	g_return_if_fail (v);
 
-	if (cell->value != NULL)
-		value_release (cell->value);
+	value_release (cell->value);
 	cell->value = v;
 }
 
@@ -439,40 +437,70 @@ gnm_cell_is_array (GnmCell const *cell)
 gboolean
 gnm_cell_is_nonsingleton_array (GnmCell const *cell)
 {
-	GnmExprArrayCorner const *corner = gnm_cell_is_array_corner (cell);
+	GnmExprArrayCorner const *corner;
 
+	if ((cell == NULL) || !gnm_cell_has_expr (cell))
+		return FALSE;
+	if (gnm_expr_top_is_array_elem (cell->base.texpr, NULL, NULL))
+		return TRUE;
+
+	corner 	= gnm_expr_top_get_array_corner (cell->base.texpr);
 	return corner && (corner->cols > 1 || corner->rows > 1);
 }
 
 /***************************************************************************/
 
+GnmRenderedValue *
+gnm_cell_get_rendered_value (GnmCell const *cell)
+{
+	g_return_val_if_fail (cell != NULL, NULL);
+
+	return gnm_rvc_query (cell->base.sheet->rendered_values, cell);
+}
+
+GnmRenderedValue *
+gnm_cell_fetch_rendered_value (GnmCell const *cell,
+			       gboolean allow_variable_width)
+{
+	GnmRenderedValue *rv;
+
+	g_return_val_if_fail (cell != NULL, NULL);
+
+	rv = gnm_cell_get_rendered_value (cell);
+	if (rv)
+		return rv;
+
+	return gnm_cell_render_value (cell, allow_variable_width);
+}
+
+void
+gnm_cell_unrender (GnmCell const *cell)
+{
+	gnm_rvc_remove (cell->base.sheet->rendered_values, cell);
+}
+
 /**
  * gnm_cell_render_value :
  * @cell: The cell whose value needs to be rendered
  * @allow_variable_width : Allow format to depend on column width.
- *
- * TODO :
- * The reason the rendered values are stored separately from the GnmCell is
- * that in the future only visible cells will be rendered.  The render
- * will be SheetControl specific to allow for multiple zooms and different
- * display resolutions.
  */
-void
-gnm_cell_render_value (GnmCell *cell, gboolean allow_variable_width)
+GnmRenderedValue *
+gnm_cell_render_value (GnmCell const *cell, gboolean allow_variable_width)
 {
 	GnmRenderedValue *rv;
 	Sheet *sheet;
 
-	g_return_if_fail (cell != NULL);
+	g_return_val_if_fail (cell != NULL, NULL);
 
 	sheet = cell->base.sheet;
-	rv = gnm_rendered_value_new (cell, gnm_cell_get_style (cell),
+	rv = gnm_rendered_value_new (cell,
+				     sheet->rendered_values->context,
 				     allow_variable_width,
-				     sheet->context,
 				     sheet->last_zoom_factor_used);
-	if (cell->rendered_value)
-		gnm_rendered_value_destroy (cell->rendered_value);
-	cell->rendered_value = rv;
+
+	gnm_rvc_store (sheet->rendered_values, cell, rv);
+
+	return rv;
 }
 
 /*
@@ -483,15 +511,15 @@ gnm_cell_render_value (GnmCell *cell, gboolean allow_variable_width)
  * then that is what you get.
  */
 char *
-gnm_cell_get_rendered_text  (GnmCell *cell)
+gnm_cell_get_rendered_text (GnmCell *cell)
 {
+	GnmRenderedValue *rv;
+
 	g_return_val_if_fail (cell != NULL, g_strdup ("ERROR"));
 
-	/* A precursor to just in time rendering Ick! */
-	if (cell->rendered_value == NULL)
-		gnm_cell_render_value (cell, TRUE);
+	rv = gnm_cell_fetch_rendered_value (cell, TRUE);
 
-	return g_strdup (gnm_rendered_value_get_text (cell->rendered_value));
+	return g_strdup (gnm_rendered_value_get_text (rv));
 }
 
 /**
@@ -504,13 +532,13 @@ gnm_cell_get_rendered_text  (GnmCell *cell)
 GOColor
 gnm_cell_get_render_color (GnmCell const *cell)
 {
-	g_return_val_if_fail (cell != NULL, 0);
+	GnmRenderedValue *rv;
 
-	/* A precursor to just in time rendering Ick! */
-	if (cell->rendered_value == NULL)
-		gnm_cell_render_value ((GnmCell *)cell, TRUE);
+	g_return_val_if_fail (cell != NULL, GO_COLOR_BLACK);
 
-	return cell->rendered_value->go_fore_color;
+	rv = gnm_cell_fetch_rendered_value (cell, TRUE);
+
+	return rv->go_fore_color;
 }
 
 /**
@@ -554,7 +582,9 @@ gnm_cell_get_entered_text (GnmCell const *cell)
 			/* Try to be reasonably smart about adding a leading quote */
 			char const *tmp = value_peek_string (v);
 
-			if (tmp[0] != '\'' && !gnm_expr_char_start_p (tmp)) {
+			if (tmp[0] != '\'' &&
+			    tmp[0] != 0 &&
+			    !gnm_expr_char_start_p (tmp)) {
 				GnmValue *val = format_match_number
 					(tmp,
 					 gnm_cell_get_format (cell),
@@ -585,11 +615,10 @@ gnm_cell_rendered_height (GnmCell const *cell)
 
 	g_return_val_if_fail (cell != NULL, 0);
 
-	rv = cell->rendered_value;
-	if (!rv)
-		return 0;
-
-	return PANGO_PIXELS (cell->rendered_value->layout_natural_height);
+	rv = gnm_cell_get_rendered_value (cell);
+	return rv
+		? PANGO_PIXELS (rv->layout_natural_height)
+		: 0;
 }
 
 /*
@@ -602,21 +631,23 @@ gnm_cell_rendered_width (GnmCell const *cell)
 
 	g_return_val_if_fail (cell != NULL, 0);
 
-	rv = cell->rendered_value;
-	if (!rv)
-		return 0;
-
-	return PANGO_PIXELS (cell->rendered_value->layout_natural_width);
+	rv = gnm_cell_get_rendered_value (cell);
+	return rv
+		? PANGO_PIXELS (rv->layout_natural_width)
+		: 0;
 }
 
 int
 gnm_cell_rendered_offset (GnmCell const * cell)
 {
-	if (!cell || !cell->rendered_value)
-		return 0;
+	const GnmRenderedValue *rv;
 
-	return (cell->rendered_value->indent_left +
-		cell->rendered_value->indent_right);
+	g_return_val_if_fail (cell != NULL, 0);
+
+	rv = gnm_cell_get_rendered_value (cell);
+	return rv
+		? rv->indent_left + rv->indent_right
+		: 0;
 }
 
 GnmStyle const *
