@@ -4,6 +4,7 @@
  *
  * Copyright (C) 2001-2006 Jody Goldberg (jody@gnome.org)
  * Copyright (C) 1998-2000 Miguel de Icaza (miguel@gnu.org)
+ * Copyright (C) 2000-2009 Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -40,12 +41,10 @@
 #include "parse-util.h"
 #include "mathfunc.h"
 
+#include <goffice/goffice.h>
 #include <math.h>
 #include <string.h>
 #include <stdlib.h>
-#include <goffice/utils/go-locale.h>
-#include <goffice/utils/go-format.h>
-#include <goffice/utils/go-glib-extras.h>
 
 /*
  * Using pools here probably does not save anything, but it's a darn
@@ -168,6 +167,23 @@ gnm_expr_new_funcall4 (GnmFunc *func,
 	return gnm_expr_new_funcallv (func, 4, argv);
 }
 
+GnmExpr const *
+gnm_expr_new_funcall5 (GnmFunc *func,
+		       GnmExpr const *arg0,
+		       GnmExpr const *arg1,
+		       GnmExpr const *arg2,
+		       GnmExpr const *arg3,
+		       GnmExpr const *arg4)
+{
+	GnmExprConstPtr *argv = g_new (GnmExprConstPtr, 5);
+	argv[0] = arg0;
+	argv[1] = arg1;
+	argv[2] = arg2;
+	argv[3] = arg3;
+	argv[4] = arg4;
+	return gnm_expr_new_funcallv (func, 5, argv);
+}
+
 
 /***************************************************************************/
 
@@ -245,6 +261,14 @@ gnm_expr_new_cellref (GnmCellRef const *cr)
 
 /***************************************************************************/
 
+static gboolean
+gnm_expr_is_array (GnmExpr const *expr)
+{
+	return expr &&
+		(GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_ARRAY_ELEM ||
+		 GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_ARRAY_CORNER);
+}
+
 /**
  * gnm_expr_new_array_corner :
  * @cols :
@@ -258,6 +282,8 @@ gnm_expr_new_array_corner(int cols, int rows, GnmExpr const *expr)
 {
 	GnmExprArrayCorner *ans;
 
+	g_return_val_if_fail (!gnm_expr_is_array (expr), NULL);
+
 	ans = CHUNK_ALLOC (GnmExprArrayCorner, expression_pool_big);
 	if (ans == NULL)
 		return NULL;
@@ -323,7 +349,7 @@ gnm_expr_new_set (GnmExprList *set)
  * @r: end range
  *
  * This function builds a range constructor or something simpler,
- * but equivalent, if the arguments allow it. 
+ * but equivalent, if the arguments allow it.
  *
  * Note: this takes ownership of @l and @r and may delete them.
  **/
@@ -476,8 +502,7 @@ gnm_expr_free (GnmExpr const *expr)
 		break;
 
 	case GNM_EXPR_OP_ARRAY_CORNER:
-		if (expr->array_corner.value)
-			value_release (expr->array_corner.value);
+		value_release (expr->array_corner.value);
 		gnm_expr_free (expr->array_corner.expr);
 		CHUNK_FREE (expression_pool_big, (gpointer)expr);
 		break;
@@ -709,7 +734,12 @@ value_intersection (GnmValue *v, GnmEvalPos const *pos)
 		int col = pos->eval.col;
 		int row = pos->eval.row;
 
-		if (r.start.row == r.end.row) {
+		if (pos->dep && !dependent_is_cell (pos->dep)) {
+			/* See bug #142412.  */
+			col = r.start.col;
+			row = r.start.row;
+			found = TRUE;
+		} else if (r.start.row == r.end.row) {
 			if (r.start.col <= col && col <= r.end.col) {
 				row = r.start.row;
 				found = TRUE;
@@ -992,8 +1022,7 @@ bin_array_iter_a (GnmEvalPos const *ep,
 	}
 
 	value_release (a);
-	if (b != NULL)
-		value_release (b);
+	value_release (b);
 	return iter_info.res;
 }
 
@@ -1025,8 +1054,7 @@ bin_array_iter_b (GnmEvalPos const *ep,
 		value_area_get_height (b, ep));
 	value_area_foreach (b, ep, CELL_ITER_ALL,
 		(GnmValueIterFunc) cb_implicit_iter_b_to_scalar_a, &iter_info);
-	if (a != NULL)
-		value_release (a);
+	value_release (a);
 	value_release (b);
 
 	return iter_info.res;
@@ -1098,8 +1126,7 @@ cb_iter_percentage (GnmValueIter const *v_iter, GnmValue *res)
 		} else
 			tmp = value_new_error_VALUE (v_iter->ep);
 
-		if (conv != NULL)
-			value_release (conv);
+		value_release (conv);
 	}
 
 	res->v_array.vals[v_iter->x][v_iter->y] = tmp;
@@ -1125,7 +1152,7 @@ gnm_expr_range_op (GnmExpr const *expr, GnmEvalPos const *ep,
 	switch (GNM_EXPR_GET_OPER (expr)) {
 	case GNM_EXPR_OP_RANGE_CTOR:
 		res_range = range_union (&a_range, &b_range);
-		break;		
+		break;
 	case GNM_EXPR_OP_INTERSECT:
 		/* 3D references not allowed.  */
 		if (a_start != a_end || b_start != b_end)
@@ -1196,8 +1223,7 @@ gnm_expr_eval (GnmExpr const *expr, GnmEvalPos const *pos,
 		b = gnm_expr_eval (expr->binary.value_b, pos, flags);
 		if (b != NULL) {
 			if (VALUE_IS_ERROR (b)) {
-				if (a != NULL)
-					value_release (a);
+				value_release (a);
 				return b;
 			}
 			if (b->type == VALUE_CELLRANGE || b->type == VALUE_ARRAY)
@@ -1207,10 +1233,8 @@ gnm_expr_eval (GnmExpr const *expr, GnmEvalPos const *pos,
 		}
 
 		res = bin_cmp (GNM_EXPR_GET_OPER (expr), value_compare (a, b, FALSE), pos);
-		if (a != NULL)
-			value_release (a);
-		if (b != NULL)
-			value_release (b);
+		value_release (a);
+		value_release (b);
 		return res;
 
 	case GNM_EXPR_OP_ADD:
@@ -1356,8 +1380,7 @@ gnm_expr_eval (GnmExpr const *expr, GnmEvalPos const *pos,
 		b = gnm_expr_eval (expr->binary.value_b, pos, flags);
 		if (b != NULL) {
 			if (VALUE_IS_ERROR (b)) {
-				if (a != NULL)
-					value_release (a);
+				value_release (a);
 				return b;
 			}
 			if (b->type == VALUE_CELLRANGE || b->type == VALUE_ARRAY)
@@ -1452,8 +1475,7 @@ gnm_expr_eval (GnmExpr const *expr, GnmEvalPos const *pos,
 		a = gnm_expr_eval (expr->array_corner.expr, &range_pos,
 			flags | GNM_EXPR_EVAL_PERMIT_NON_SCALAR);
 
-		if (expr->array_corner.value)
-			value_release (expr->array_corner.value);
+		value_release (expr->array_corner.value);
 
 		/* Store real result (cast away const)*/
 		((GnmExpr*)expr)->array_corner.value = a;
@@ -1527,11 +1549,6 @@ gnm_expr_eval (GnmExpr const *expr, GnmEvalPos const *pos,
 	return value_new_error (pos, _("Unknown evaluation error"));
 }
 
-static void
-gnm_expr_list_as_string (int argc, GnmExprConstPtr const *argv,
-			 GnmConventionsOut *out);
-
-
 /*
  * Converts a parsed tree into its string representation
  * assuming that we are evaluating at col, row
@@ -1560,7 +1577,7 @@ do_expr_as_string (GnmExpr const *expr, int paren_level,
 		{ "-",  3, 1, 0, 0 },
 		{ "*",  4, 1, 0, 0 },
 		{ "/",  4, 1, 0, 0 },
-		{ "^",  5, 0, 1, 0 },
+		{ "^",  5, 0, 0, 0 }, /* Note: neither left nor right */
 		{ "&",  2, 1, 0, 0 },
 		{ "",   0, 0, 0, 0 }, /* Funcall  */
 		{ "",   0, 0, 0, 0 }, /* Name     */
@@ -1634,15 +1651,9 @@ do_expr_as_string (GnmExpr const *expr, int paren_level,
 		return;
 	}
 
-	case GNM_EXPR_OP_FUNCALL: {
-		GnmExprFunction const *func = &expr->func;
-		char const *name = gnm_func_get_name (func->func);
-
-		g_string_append (target, name);
-		/* FIXME: possibly a space here.  */
-		gnm_expr_list_as_string (func->argc, func->argv, out);
+	case GNM_EXPR_OP_FUNCALL:
+		out->convs->output.func (out, &expr->func);
 		return;
-	}
 
 	case GNM_EXPR_OP_NAME:
 		out->convs->output.name (out, &expr->name);
@@ -1727,7 +1738,6 @@ gnm_expr_as_string (GnmExpr const *expr, GnmParsePos const *pp,
 	GnmParsePos pp0;
 
 	g_return_val_if_fail (expr != NULL, NULL);
-	g_return_val_if_fail (pp != NULL, NULL);
 
 	/*
 	 * Defaults for debugging only!
@@ -1750,6 +1760,19 @@ gnm_expr_as_string (GnmExpr const *expr, GnmParsePos const *pp,
 }
 
 /****************************************************************************/
+
+static gboolean
+gnm_expr_is_err (GnmExpr const *expr, GnmStdError err)
+{
+	GnmStdError err2;
+
+	if (GNM_EXPR_GET_OPER (expr) != GNM_EXPR_OP_CONSTANT)
+		return FALSE;
+
+	err2 = value_error_classify (expr->constant.value);
+	return err == err2;
+}
+
 typedef struct {
 	GnmExprRelocateInfo const *details;
 	gboolean from_inside;
@@ -1886,14 +1909,17 @@ reloc_normalize_cellref (RelocInfoInternal const *rinfo, GnmCellRef const *ref,
 /* Return TRUE if @pos is out of bounds */
 static gboolean
 reloc_restore_cellref (RelocInfoInternal const *rinfo,
-		       Sheet *sheet, GnmCellPos const *pos,
+		       GnmSheetSize const *ss, GnmCellPos const *pos,
 		       GnmCellRef *res)
 {
-	if (res->sheet == rinfo->details->origin_sheet)
+	if (res->sheet == rinfo->details->origin_sheet) {
 		res->sheet = rinfo->details->target_sheet;
+		if (res->sheet)
+			ss = gnm_sheet_get_size (res->sheet);
+	}
 
-	if (!res->col_relative  || rinfo->check_rels) {
-		if (pos->col < 0 || gnm_sheet_get_max_cols (sheet) <= pos->col)
+	if (!res->col_relative || rinfo->check_rels) {
+		if (pos->col < 0 || ss->max_cols <= pos->col)
 			return TRUE;
 		res->col = pos->col;
 		if (res->col_relative) {
@@ -1902,8 +1928,9 @@ reloc_restore_cellref (RelocInfoInternal const *rinfo,
 				res->col -= rinfo->details->col_offset;
 		}
 	}
-	if (!res->row_relative  || rinfo->check_rels) {
-		if (pos->row < 0 || gnm_sheet_get_max_rows (sheet) <= pos->row)
+
+	if (!res->row_relative || rinfo->check_rels) {
+		if (pos->row < 0 || ss->max_rows <= pos->row)
 			return TRUE;
 		res->row = pos->row;
 		if (res->row_relative) {
@@ -1923,7 +1950,9 @@ reloc_cellrange (RelocInfoInternal const *rinfo, GnmValueRange const *v,
 {
 	GnmRange r;
 	Sheet   *start_sheet, *end_sheet;
+	GnmSheetSize const *start_ss, *end_ss;
 	gboolean full_col, full_row;
+	gboolean full_col_begin, full_row_begin;
 
 	/* Normalize the rangeRef, and remember if we had a full col/row
 	 *  ref.  If relocating the result changes things, or if we're from
@@ -1935,21 +1964,30 @@ reloc_cellrange (RelocInfoInternal const *rinfo, GnmValueRange const *v,
 	/* (Foo,NULL) in Bar will generate (Foo,Bar) in normalize */
 	if (NULL == v->cell.b.sheet)
 		end_sheet = start_sheet;
+	start_ss = gnm_sheet_get_size2 (start_sheet, rinfo->details->pos.wb);
+	end_ss = gnm_sheet_get_size2 (end_sheet, rinfo->details->pos.wb);
 
-	full_col = sticky_end && r.end.row >= gnm_sheet_get_last_row (start_sheet);
-	full_row = sticky_end && r.end.col >= gnm_sheet_get_last_col (start_sheet);
+	full_col = sticky_end && r.end.row >= start_ss->max_rows - 1;
+	full_col_begin = full_col && r.start.row == 0;
+
+	full_row = sticky_end && r.end.col >= start_ss->max_cols - 1;
+	full_row_begin = full_row && r.start.col == 0;
 
 	if (reloc_range (rinfo->details, start_sheet, end_sheet, &r) ||
 	    rinfo->from_inside) {
 		GnmRangeRef res = v->cell;
 
 		if (full_col)
-			r.end.row = gnm_sheet_get_last_row (start_sheet);
+			r.end.row = start_ss->max_rows - 1;
+		if (full_col_begin)
+			r.start.row = 0;
 		if (full_row)
-			r.end.col = gnm_sheet_get_last_col (start_sheet);
+			r.end.col = start_ss->max_cols - 1;
+		if (full_row_begin)
+			r.start.col = 0;
 
-		if (reloc_restore_cellref (rinfo, start_sheet, &r.start, &res.a) ||
-		    reloc_restore_cellref (rinfo, end_sheet,   &r.end,   &res.b))
+		if (reloc_restore_cellref (rinfo, start_ss, &r.start, &res.a) ||
+		    reloc_restore_cellref (rinfo, end_ss,   &r.end,   &res.b))
 			return gnm_expr_new_constant (value_new_error_REF (NULL));
 		if (gnm_rangeref_equal (&res, &v->cell))
 			return NULL;
@@ -2082,16 +2120,19 @@ gnm_expr_relocate (GnmExpr const *expr, RelocInfoInternal const *rinfo)
 			} else {
 				/*
 				 * Do NOT rewrite the name.
-				 * Just invalidate the use of the name
+				 *
+				 * Just invalidate the use of the name if the
+				 * name's expression, if relocated, would
+				 * become invalid.
 				 */
 				GnmExpr const *tmp =
 					gnm_expr_relocate (nexpr->texpr->expr,
 							   rinfo);
-				if (tmp != NULL) {
+				if (tmp && gnm_expr_is_err (tmp, GNM_ERROR_REF))
+					return tmp;
+
+				if (tmp)
 					gnm_expr_free (tmp);
-					return gnm_expr_new_constant (
-						value_new_error_REF (NULL));
-				}
 
 				return NULL;
 			}
@@ -2118,15 +2159,17 @@ gnm_expr_relocate (GnmExpr const *expr, RelocInfoInternal const *rinfo)
 		case GNM_EXPR_RELOCATE_COLS:
 		case GNM_EXPR_RELOCATE_ROWS: {
 			GnmRange r;
-			Sheet   *sheet;
+			Sheet *sheet;
+			GnmSheetSize const *ss;
 
 			reloc_normalize_cellref (rinfo, ref, &sheet, &r.start);
 			r.end = r.start;
+			ss = gnm_sheet_get_size2 (sheet, rinfo->details->pos.wb);
 
 			if (reloc_range (rinfo->details, sheet, sheet, &r) ||
 			    rinfo->from_inside) {
 				GnmCellRef res = *ref;
-				if (reloc_restore_cellref (rinfo, sheet, &r.start, &res))
+				if (reloc_restore_cellref (rinfo, ss, &r.start, &res))
 					return gnm_expr_new_constant (value_new_error_REF (NULL));
 				if (gnm_cellref_equal (&res, ref))
 					return NULL;
@@ -2223,11 +2266,17 @@ gnm_expr_first_funcall (GnmExpr const *expr)
 }
 
 static void
-cellref_boundingbox (GnmCellRef const *cr, GnmRange *bound)
+cellref_boundingbox (GnmCellRef const *cr, Sheet const *sheet, GnmRange *bound)
 {
+	GnmSheetSize const *ss;
+
+	if (cr->sheet)
+		sheet = cr->sheet;
+	ss = gnm_sheet_get_size (sheet);
+
 	if (cr->col_relative) {
 		if (cr->col >= 0) {
-			int const c = gnm_sheet_get_max_cols (cr->sheet) - cr->col - 1;
+			int const c = ss->max_cols - cr->col - 1;
 			if (bound->end.col > c)
 				bound->end.col = c;
 		} else {
@@ -2238,7 +2287,7 @@ cellref_boundingbox (GnmCellRef const *cr, GnmRange *bound)
 	}
 	if (cr->row_relative) {
 		if (cr->row >= 0) {
-			int const r = gnm_sheet_get_max_rows (cr->sheet) - cr->row - 1;
+			int const r = ss->max_rows - cr->row - 1;
 			if (bound->end.row > r)
 				bound->end.row = r;
 		} else {
@@ -2250,9 +2299,9 @@ cellref_boundingbox (GnmCellRef const *cr, GnmRange *bound)
 }
 
 static GSList *
-g_slist_insert_unique (GSList *list, gpointer data)
+gnm_insert_unique (GSList *list, gpointer data)
 {
-	if (data != NULL && g_slist_find (list, data) == NULL)
+	if (g_slist_find (list, data) == NULL)
 		return g_slist_prepend (list, data);
 	return list;
 }
@@ -2264,11 +2313,8 @@ do_referenced_sheets (GnmExpr const *expr, GSList *sheets)
 	case GNM_EXPR_OP_RANGE_CTOR:
 	case GNM_EXPR_OP_INTERSECT:
 	case GNM_EXPR_OP_ANY_BINARY:
-		return do_referenced_sheets (
-			expr->binary.value_a,
-			do_referenced_sheets (
-				expr->binary.value_b,
-				sheets));
+		sheets = do_referenced_sheets (expr->binary.value_a, sheets);
+		return do_referenced_sheets (expr->binary.value_b, sheets);
 
 	case GNM_EXPR_OP_ANY_UNARY:
 		return do_referenced_sheets (expr->unary.value, sheets);
@@ -2292,16 +2338,17 @@ do_referenced_sheets (GnmExpr const *expr, GSList *sheets)
 		return sheets;
 
 	case GNM_EXPR_OP_CELLREF:
-		return g_slist_insert_unique (sheets, expr->cellref.ref.sheet);
+		return gnm_insert_unique (sheets, expr->cellref.ref.sheet);
 
 	case GNM_EXPR_OP_CONSTANT: {
 		GnmValue const *v = expr->constant.value;
 		if (v->type != VALUE_CELLRANGE)
 			return sheets;
-		return g_slist_insert_unique (
-			g_slist_insert_unique (sheets,
-					       v->v_range.cell.a.sheet),
-			v->v_range.cell.b.sheet);
+		sheets = gnm_insert_unique (sheets, v->v_range.cell.a.sheet);
+		/* A NULL b sheet means a's sheet.  Do not insert that.  */
+		if (v->v_range.cell.b.sheet)
+			sheets = gnm_insert_unique (sheets, v->v_range.cell.b.sheet);
+		return sheets;
 	}
 
 	case GNM_EXPR_OP_ARRAY_CORNER:
@@ -2365,7 +2412,8 @@ gnm_expr_contains_subtotal (GnmExpr const *expr)
 }
 
 static void
-gnm_expr_get_boundingbox (GnmExpr const *expr, GnmRange *bound)
+gnm_expr_get_boundingbox (GnmExpr const *expr, Sheet const *sheet,
+			  GnmRange *bound)
 {
 	g_return_if_fail (expr != NULL);
 
@@ -2373,24 +2421,24 @@ gnm_expr_get_boundingbox (GnmExpr const *expr, GnmRange *bound)
 	case GNM_EXPR_OP_RANGE_CTOR:
 	case GNM_EXPR_OP_INTERSECT:
 	case GNM_EXPR_OP_ANY_BINARY:
-		gnm_expr_get_boundingbox (expr->binary.value_a, bound);
-		gnm_expr_get_boundingbox (expr->binary.value_b, bound);
+		gnm_expr_get_boundingbox (expr->binary.value_a, sheet, bound);
+		gnm_expr_get_boundingbox (expr->binary.value_b, sheet, bound);
 		break;
 
 	case GNM_EXPR_OP_ANY_UNARY:
-		gnm_expr_get_boundingbox (expr->unary.value, bound);
+		gnm_expr_get_boundingbox (expr->unary.value, sheet, bound);
 		break;
 
 	case GNM_EXPR_OP_FUNCALL: {
 		int i;
 		for (i = 0; i < expr->func.argc; i++)
-			gnm_expr_get_boundingbox (expr->func.argv[i], bound);
+			gnm_expr_get_boundingbox (expr->func.argv[i], sheet, bound);
 		break;
 	}
 	case GNM_EXPR_OP_SET: {
 		int i;
 		for (i = 0; i < expr->set.argc; i++)
-			gnm_expr_get_boundingbox (expr->set.argv[i], bound);
+			gnm_expr_get_boundingbox (expr->set.argv[i], sheet, bound);
 		break;
 	}
 
@@ -2400,21 +2448,21 @@ gnm_expr_get_boundingbox (GnmExpr const *expr, GnmRange *bound)
 		break;
 
 	case GNM_EXPR_OP_CELLREF:
-		cellref_boundingbox (&expr->cellref.ref, bound);
+		cellref_boundingbox (&expr->cellref.ref, sheet, bound);
 		break;
 
 	case GNM_EXPR_OP_CONSTANT: {
 		GnmValue const *v = expr->constant.value;
 
 		if (v->type == VALUE_CELLRANGE) {
-			cellref_boundingbox (&v->v_range.cell.a, bound);
-			cellref_boundingbox (&v->v_range.cell.b, bound);
+			cellref_boundingbox (&v->v_range.cell.a, sheet, bound);
+			cellref_boundingbox (&v->v_range.cell.b, sheet, bound);
 		}
 		break;
 	}
 
 	case GNM_EXPR_OP_ARRAY_CORNER:
-		gnm_expr_get_boundingbox (expr->array_corner.expr, bound);
+		gnm_expr_get_boundingbox (expr->array_corner.expr, sheet, bound);
 		break;
 
 	case GNM_EXPR_OP_ARRAY_ELEM:
@@ -2436,7 +2484,7 @@ gnm_expr_get_range (GnmExpr const *expr)
 	g_return_val_if_fail (expr != NULL, NULL);
 
 	switch (GNM_EXPR_GET_OPER (expr)) {
-	case GNM_EXPR_OP_CELLREF :
+	case GNM_EXPR_OP_CELLREF:
 		return value_new_cellrange_unsafe (
 			&expr->cellref.ref, &expr->cellref.ref);
 
@@ -2450,6 +2498,9 @@ gnm_expr_get_range (GnmExpr const *expr)
 			return NULL;
 		return gnm_expr_top_get_range (expr->name.name->texpr);
 
+	case GNM_EXPR_OP_PAREN:
+		return gnm_expr_get_range (expr->unary.value);
+
 	default:
 		return NULL;
 	}
@@ -2491,7 +2542,7 @@ do_gnm_expr_get_ranges (GnmExpr const *expr, GSList *ranges)
 	default: {
 		GnmValue *v = gnm_expr_get_range (expr);
 		if (v)
-			return g_slist_insert_unique (ranges, v);
+			return gnm_insert_unique (ranges, v);
 		return ranges;
 	}
 	}
@@ -2576,6 +2627,15 @@ gnm_expr_is_data_table (GnmExpr const *expr, GnmCellPos *c_in, GnmCellPos *r_in)
 	return FALSE;
 }
 
+gboolean
+gnm_expr_is_empty (GnmExpr const *expr)
+{
+	g_return_val_if_fail (expr != NULL, FALSE);
+
+	return (GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_CONSTANT &&
+		VALUE_IS_EMPTY (expr->constant.value));
+}
+
 /*
  * This frees the data pointers and the list.
  */
@@ -2588,7 +2648,7 @@ gnm_expr_list_unref (GnmExprList *list)
 	gnm_expr_list_free (list);
 }
 
-static void
+void
 gnm_expr_list_as_string (int argc,
 			 GnmExprConstPtr const *argv,
 			 GnmConventionsOut *out)
@@ -2844,6 +2904,8 @@ gnm_expr_top_equal (GnmExprTop const *te1, GnmExprTop const *te2)
 {
 	if (te1 == te2)
 		return TRUE;
+	if (te1 == NULL || te2 == NULL)
+		return FALSE;
 
 	g_return_val_if_fail (IS_GNM_EXPR_TOP (te1), FALSE);
 	g_return_val_if_fail (IS_GNM_EXPR_TOP (te2), FALSE);
@@ -2940,18 +3002,23 @@ gnm_expr_top_eval (GnmExprTop const *texpr,
 		   GnmEvalPos const *pos,
 		   GnmExprEvalFlags flags)
 {
+	GnmValue *res;
 	g_return_val_if_fail (IS_GNM_EXPR_TOP (texpr), NULL);
 
-	return gnm_expr_eval (texpr->expr, pos, flags);
+	gnm_app_recalc_start ();
+	res = gnm_expr_eval (texpr->expr, pos, flags);
+	gnm_app_recalc_finish ();
+
+	return res;
 }
 
 /**
  * gnm_expr_top_referenced_sheets :
  * @texpr :
- * @sheets : usually NULL.
  *
  * Generates a list of the sheets referenced by the supplied expression.
- * Caller must free the list.
+ * Caller must free the list.  Note, that NULL may occur in the result
+ * if the expression has a range or cellref without a sheet.
  */
 GSList *
 gnm_expr_top_referenced_sheets (GnmExprTop const *texpr)
@@ -2964,14 +3031,8 @@ gnm_expr_top_referenced_sheets (GnmExprTop const *texpr)
 gboolean
 gnm_expr_top_is_err (GnmExprTop const *texpr, GnmStdError err)
 {
-	GnmStdError err2;
 	g_return_val_if_fail (IS_GNM_EXPR_TOP (texpr), FALSE);
-
-	if (GNM_EXPR_GET_OPER (texpr->expr) != GNM_EXPR_OP_CONSTANT)
-		return FALSE;
-
-	err2 = value_error_classify (texpr->expr->constant.value);
-	return err == err2;
+	return gnm_expr_is_err (texpr->expr, err);
 }
 
 /**
@@ -2991,6 +3052,17 @@ gnm_expr_top_get_constant (GnmExprTop const *texpr)
 	return texpr->expr->constant.value;
 }
 
+GnmCellRef const *
+gnm_expr_top_get_cellref (GnmExprTop const *texpr)
+{
+	g_return_val_if_fail (IS_GNM_EXPR_TOP (texpr), NULL);
+
+	if (GNM_EXPR_GET_OPER (texpr->expr) != GNM_EXPR_OP_CELLREF)
+		return NULL;
+
+	return &texpr->expr->cellref.ref;
+}
+
 /**
  * gnm_expr_top_first_funcall :
  * @texpr :
@@ -3011,11 +3083,12 @@ gnm_expr_top_first_funcall (GnmExprTop const *texpr)
  * out of bounds.
  **/
 void
-gnm_expr_top_get_boundingbox (GnmExprTop const *texpr, GnmRange *bound)
+gnm_expr_top_get_boundingbox (GnmExprTop const *texpr, Sheet const *sheet,
+			      GnmRange *bound)
 {
 	g_return_if_fail (IS_GNM_EXPR_TOP (texpr));
 
-	gnm_expr_get_boundingbox (texpr->expr, bound);
+	gnm_expr_get_boundingbox (texpr->expr, sheet, bound);
 }
 
 gboolean
@@ -3059,8 +3132,7 @@ gboolean
 gnm_expr_top_is_array (GnmExprTop const *texpr)
 {
 	g_return_val_if_fail (IS_GNM_EXPR_TOP (texpr), FALSE);
-	return (GNM_EXPR_GET_OPER (texpr->expr) == GNM_EXPR_OP_ARRAY_ELEM ||
-		GNM_EXPR_GET_OPER (texpr->expr) == GNM_EXPR_OP_ARRAY_CORNER);
+	return gnm_expr_is_array (texpr->expr);
 }
 
 GnmExprTop const *
@@ -3148,7 +3220,7 @@ cb_expression_pool_leak (gpointer data, G_GNUC_UNUSED gpointer user)
 	pp.eval.row = 0;
 	pp.sheet = NULL;
 	pp.wb = NULL;
-	s = gnm_expr_as_string (expr, &pp, gnm_conventions_default);
+	s = gnm_expr_as_string (expr, &pp, NULL);
 	g_printerr ("Leaking expression at %p: %s.\n", (void *)expr, s);
 	g_free (s);
 }
