@@ -55,9 +55,11 @@ struct _ItemEdit {
 	GnmCellPos pos;
 	gboolean   cursor_visible;
 	int        blink_timer;
+	int	   sel_start;
 
 	GnmFont   *gfont;
 	GnmStyle  *style;
+	GdkGC    *gc;
 };
 
 typedef GocItemClass ItemEditClass;
@@ -95,6 +97,9 @@ item_edit_draw (GocItem const *item, cairo_t *cr)
 	int top, left;
 	GOColor color;
 	int x0, y0, x1, y1; /* in widget coordinates */
+	int start, end;
+	PangoRectangle pos;
+	char const *text = gtk_entry_get_text (ie->entry);
 
 	get_top_left (ie, &top, &left);
 	if (goc_canvas_get_direction (item->canvas) == GOC_DIRECTION_RTL) {
@@ -106,16 +111,16 @@ item_edit_draw (GocItem const *item, cairo_t *cr)
 	}
 
 	cairo_rectangle (cr, x0, y0, x1 - x0, y1 - y0);
-	cairo_set_source_rgba (cr, 1., 1., 0.878431373, 1.);
+	if (!gnumeric_background_set (ie->style, cr, FALSE))
+		cairo_set_source_rgba (cr, 1., 1., 0.878431373, 1.);
 	cairo_fill (cr);
 
 	color = GO_COLOR_FROM_GDK (gtk_widget_get_style (GTK_WIDGET (item->canvas))->black);
 	cairo_set_source_rgba (cr, GO_COLOR_TO_CAIRO (color));
 	cairo_move_to (cr, left, top);
+	gtk_editable_get_selection_bounds (GTK_EDITABLE (ie->entry), &start, &end);
 	pango_cairo_show_layout (cr, ie->layout);
 	if (ie->cursor_visible) {
-		PangoRectangle pos;
-		char const *text = gtk_entry_get_text (ie->entry);
 		int cursor_pos = gtk_editable_get_position (GTK_EDITABLE (ie->entry));
 		pango_layout_index_to_pos (ie->layout,
 			g_utf8_offset_to_pointer (text, cursor_pos) - text, &pos);
@@ -123,11 +128,30 @@ item_edit_draw (GocItem const *item, cairo_t *cr)
 		cairo_set_dash (cr, NULL, 0, 0.);
 		cairo_set_line_cap (cr, CAIRO_LINE_CAP_BUTT);
 		cairo_set_line_join (cr, CAIRO_LINE_JOIN_MITER);
-		cairo_set_source_rgba (cr, 0., 0., 0., 1.);
+		cairo_set_source_rgba (cr, GO_COLOR_TO_CAIRO (gnm_style_get_back_color (ie->style)->go_color ^ 0xffffff00));
 		cairo_move_to (cr, left + PANGO_PIXELS (pos.x) + .5, top + PANGO_PIXELS (pos.y));
 		cairo_line_to (cr, left + PANGO_PIXELS (pos.x) + .5, top + PANGO_PIXELS (pos.y + pos.height) - 1);
 		cairo_stroke (cr);
 	}
+	if (start != end) {
+		/* invert selected region */
+		int x, y, w, h;
+		GdkEventExpose *expose = (GdkEventExpose *) goc_canvas_get_cur_event (item->canvas);
+		GdkDrawable *drawable = GDK_DRAWABLE (expose->window);
+		start = g_utf8_offset_to_pointer (text, start) - text;
+		end = g_utf8_offset_to_pointer (text, end) - text;
+		pango_layout_index_to_pos (ie->layout, start, &pos);
+		x = PANGO_PIXELS (pos.x);
+		y = PANGO_PIXELS (pos.y);
+		h = PANGO_PIXELS (pos.height);
+		pango_layout_index_to_pos (ie->layout, end, &pos);
+		w = PANGO_PIXELS (pos.x) - x;
+		if (w < 0) {
+			x += w;
+			w = -w;
+		}
+		gdk_draw_rectangle (drawable, ie->gc, TRUE, left + x, top + y, w, h);
+	}
 }
 
 static double
@@ -157,6 +181,7 @@ item_edit_button_pressed (GocItem *item, int button, double x, double y)
 		GtkEditable *ed = GTK_EDITABLE (ie->entry);
 		int target_index, trailing;
 		int top, left;
+		char const *text = pango_layout_get_text (ie->layout);
 
 		get_top_left (ie, &top, &left);
 		y -= top;
@@ -166,7 +191,6 @@ item_edit_button_pressed (GocItem *item, int button, double x, double y)
 					      x * PANGO_SCALE, y * PANGO_SCALE,
 					      &target_index, &trailing)) {
 			int preedit = GNM_PANE (item->canvas)->preedit_length;
-			char const *text = pango_layout_get_text (ie->layout);
 			gint cur_index = gtk_editable_get_position (ed);
 			cur_index = g_utf8_offset_to_pointer (text, cur_index) - text;
 
@@ -177,12 +201,72 @@ item_edit_button_pressed (GocItem *item, int button, double x, double y)
 				} else
 					target_index -= preedit;
 			}
-			gtk_editable_set_position (GTK_EDITABLE (ie->entry),
-				g_utf8_pointer_to_offset (text, text + target_index)
-				+ trailing);
-
-			return TRUE;
+		} else {
+			/* the click occured after text end (#388342) */
+			target_index = strlen (text);
+			trailing = 0;
 		}
+		ie->sel_start = g_utf8_pointer_to_offset (text, text + target_index) + trailing; 
+		gtk_editable_set_position (GTK_EDITABLE (ie->entry), ie->sel_start);
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static gboolean
+item_edit_motion (GocItem *item, double x, double y)
+{
+	ItemEdit *ie = ITEM_EDIT (item);
+	if (ie->sel_start >=0) {
+		GtkEditable *ed = GTK_EDITABLE (ie->entry);
+		int target_index, trailing;
+		int top, left;
+		char const *text = pango_layout_get_text (ie->layout);
+
+		get_top_left (ie, &top, &left);
+		y -= top;
+		x -= left;
+
+		if (pango_layout_xy_to_index (ie->layout,
+					      x * PANGO_SCALE, y * PANGO_SCALE,
+					      &target_index, &trailing)) {
+			int preedit = GNM_PANE (item->canvas)->preedit_length;
+			gint cur_index = gtk_editable_get_position (ed);
+			cur_index = g_utf8_offset_to_pointer (text, cur_index) - text;
+
+			if (target_index >= cur_index && preedit > 0) {
+				if (target_index < (cur_index + preedit)) {
+					target_index = cur_index;
+					trailing = 0;
+				} else
+					target_index -= preedit;
+			}
+		} else {
+			/* the click occured after text end (#388342) */
+			target_index = strlen (text);
+			trailing = 0;
+		}
+		target_index = g_utf8_pointer_to_offset (text, text + target_index) + trailing;
+		if (target_index > ie->sel_start)
+			gtk_editable_select_region (GTK_EDITABLE (ie->entry), ie->sel_start, target_index);
+		else
+			gtk_editable_select_region (GTK_EDITABLE (ie->entry), target_index, ie->sel_start);
+		goc_item_invalidate (item);
+
+		return TRUE;
+	}
+	return FALSE;
+}
+
+static gboolean
+item_edit_button_released (GocItem *item, int button, G_GNUC_UNUSED double x, G_GNUC_UNUSED double y)
+{
+	ItemEdit *ie = ITEM_EDIT (item);
+	if (ie->sel_start >= 0) {
+		ie->sel_start = -1;
+		return TRUE;
 	}
 	return FALSE;
 }
@@ -241,8 +325,6 @@ item_edit_update_bounds (GocItem *item)
 		pango_layout_set_attributes (ie->layout, attrs);
 		pango_attr_list_unref (attrs);
 
-		text = wbcg_edit_get_display_text (scg_wbcg (ie->scg));
-
 		if (pane->preedit_length) {
 			PangoAttrList *tmp_attrs = pango_attr_list_new ();
 			pango_attr_list_splice (tmp_attrs, pane->preedit_attrs,
@@ -359,6 +441,7 @@ item_edit_realize (GocItem *item)
 	Sheet const *sheet;
 	GnmPane	*pane;
 	double scale;
+	GdkGCValues values;
 
 	parent_class->realize (item);
 
@@ -366,7 +449,7 @@ item_edit_realize (GocItem *item)
 
 	g_signal_connect_object (G_OBJECT (scg_wbcg (ie->scg)),
 		"markup-changed",
-		G_CALLBACK (goc_item_invalidate), G_OBJECT (ie),
+		G_CALLBACK (goc_item_bounds_changed), G_OBJECT (ie),
 		G_CONNECT_SWAPPED);
 
 	g_signal_connect_object (G_OBJECT (gtk_widget_get_parent (GTK_WIDGET (ie->entry))),
@@ -388,7 +471,9 @@ item_edit_realize (GocItem *item)
 	scale = item->canvas->pixels_per_unit;
 	ie->style = gnm_style_dup
 		(sheet_style_get (sheet, ie->pos.col, ie->pos.row));
-	ie->gfont = gnm_style_get_font (ie->style, sheet->context);
+	ie->gfont = gnm_style_get_font
+		(ie->style,
+		 gtk_widget_get_pango_context (GTK_WIDGET (pane)));
 	gnm_font_ref (ie->gfont);
 
 	if (gnm_style_get_align_h (ie->style) == HALIGN_GENERAL)
@@ -416,6 +501,12 @@ item_edit_realize (GocItem *item)
 				    : PANGO_ALIGN_LEFT);
 
 	item_edit_cursor_blink_start (ie);
+
+	ie->gc = gdk_gc_new (GTK_WIDGET (item->canvas)->window);
+	gdk_gc_set_rgb_fg_color (ie->gc, &gs_white);
+	gdk_gc_set_rgb_bg_color (ie->gc, &gs_white);
+	values.function = GDK_XOR;
+	gdk_gc_set_values (ie->gc, &values, GDK_GC_FUNCTION);
 }
 
 static void
@@ -423,6 +514,9 @@ item_edit_unrealize (GocItem *item)
 {
 	ItemEdit *ie = ITEM_EDIT (item);
 
+	g_object_unref (G_OBJECT (ie->gc));
+	ie->gc = NULL;
+
 	item_edit_cursor_blink_stop (ie);
 
 	/* to destroy the feedback ranges */
@@ -455,6 +549,7 @@ item_edit_init (ItemEdit *ie)
 	ie->gfont = NULL;
 	ie->style      = NULL;
 	ie->cursor_visible = TRUE;
+	ie->sel_start = -1;
 }
 
 static void
@@ -499,13 +594,15 @@ item_edit_class_init (GObjectClass *gobject_class)
 			 GSF_PARAM_STATIC | G_PARAM_WRITABLE));
 
 	/* GocItem method overrides */
-	item_class->realize        = item_edit_realize;
-	item_class->unrealize      = item_edit_unrealize;
-	item_class->draw           = item_edit_draw;
-	item_class->distance       = item_edit_distance;
-	item_class->update_bounds  = item_edit_update_bounds;
-	item_class->button_pressed = item_edit_button_pressed;
-	item_class->enter_notify   = item_edit_enter_notify;
+	item_class->realize         = item_edit_realize;
+	item_class->unrealize       = item_edit_unrealize;
+	item_class->draw            = item_edit_draw;
+	item_class->distance        = item_edit_distance;
+	item_class->update_bounds   = item_edit_update_bounds;
+	item_class->button_pressed  = item_edit_button_pressed;
+	item_class->enter_notify    = item_edit_enter_notify;
+	item_class->motion          = item_edit_motion;
+	item_class->button_released = item_edit_button_released;
 }
 
 GSF_CLASS (ItemEdit, item_edit,
