@@ -62,6 +62,7 @@
 #include "gnumeric-gconf.h"
 #include "expr.h"
 #include "print.h"
+#include "print-info.h"
 #include "gnm-pane-impl.h"
 
 #include <goffice/goffice.h>
@@ -890,6 +891,7 @@ static GNM_ACTION_DEF (cb_tools_regression)	{ dialog_regression_tool (wbcg, wbcg
 static GNM_ACTION_DEF (cb_tools_sampling)	{ dialog_sampling_tool (wbcg, wbcg_cur_sheet (wbcg)); }
 static GNM_ACTION_DEF (cb_tools_sign_test_one_median)	{ dialog_sign_test_tool (wbcg, wbcg_cur_sheet (wbcg), SIGNTEST_1); }
 static GNM_ACTION_DEF (cb_tools_sign_test_two_medians)	{ dialog_sign_test_tool (wbcg, wbcg_cur_sheet (wbcg), SIGNTEST_2); }
+static GNM_ACTION_DEF (cb_tools_wilcoxon_mann_whitney)	{ dialog_wilcoxon_m_w_tool (wbcg, wbcg_cur_sheet (wbcg)); }
 static GNM_ACTION_DEF (cb_tools_ttest_paired)	{ dialog_ttest_tool (wbcg, wbcg_cur_sheet (wbcg), TTEST_PAIRED); }
 static GNM_ACTION_DEF (cb_tools_ttest_equal_var) { dialog_ttest_tool (wbcg, wbcg_cur_sheet (wbcg), TTEST_UNPAIRED_EQUALVARIANCES); }
 static GNM_ACTION_DEF (cb_tools_ttest_unequal_var) { dialog_ttest_tool (wbcg, wbcg_cur_sheet (wbcg), TTEST_UNPAIRED_UNEQUALVARIANCES); }
@@ -902,7 +904,7 @@ static GNM_ACTION_DEF (cb_data_shuffle)		{ dialog_shuffle (wbcg); }
 static GNM_ACTION_DEF (cb_data_import_text)	{ gui_file_open (wbcg, "Gnumeric_stf:stf_assistant"); }
 
 static GNM_ACTION_DEF (cb_auto_filter)          { cmd_autofilter_add_remove (WORKBOOK_CONTROL (wbcg)); }
-static GNM_ACTION_DEF (cb_show_all)		{ filter_show_all (wbcg_cur_sheet (wbcg)); }
+static GNM_ACTION_DEF (cb_show_all)		{ filter_show_all (WORKBOOK_CONTROL (wbcg)); }
 static GNM_ACTION_DEF (cb_data_filter)		{ dialog_advanced_filter (wbcg); }
 static GNM_ACTION_DEF (cb_data_validate)	{ dialog_cell_format (wbcg, FD_VALIDATION); }
 static GNM_ACTION_DEF (cb_data_text_to_columns) { stf_text_to_columns (WORKBOOK_CONTROL (wbcg), GO_CMD_CONTEXT (wbcg)); }
@@ -1180,6 +1182,26 @@ static GNM_ACTION_DEF (cb_sort_descending) { sort_by_rows (wbcg, TRUE); }
 static void
 cb_add_graph (GogGraph *graph, gpointer wbcg)
 {
+	GraphDataClosure *data = (GraphDataClosure *) g_object_get_data (G_OBJECT (graph), "data-closure");
+	if (data) {
+		if (data->new_sheet) {
+			WorkbookControl *wbc = WORKBOOK_CONTROL (wbcg);
+			Sheet *sheet = wb_control_cur_sheet (wbc);
+			WorkbookSheetState *old_state = workbook_sheet_state_new (wb_control_get_workbook (wbc));
+			Sheet *new_sheet = workbook_sheet_add_with_type (
+				wb_control_get_workbook (wbc),
+				GNM_SHEET_OBJECT, -1,
+				gnm_sheet_get_max_cols (sheet),
+				gnm_sheet_get_max_rows (sheet));
+			SheetObject *sog = sheet_object_graph_new (graph);
+			print_info_set_paper_orientation (new_sheet->print_info, GTK_PAGE_ORIENTATION_LANDSCAPE);
+			sheet_object_set_sheet (sog, new_sheet);
+			wb_view_sheet_focus (wb_control_view (wbc), new_sheet);
+			cmd_reorganize_sheets (wbc, old_state, sheet);
+			g_object_unref (sog);
+			return;
+		}
+	}
 	wbcg_insert_object (WBC_GTK (wbcg), sheet_object_graph_new (graph));
 }
 
@@ -1704,10 +1726,8 @@ static GtkActionEntry const permanent_actions[] = {
 		{ "MenuViewToolbars",		NULL, N_("_Toolbars") },
 	{ "MenuInsert",		NULL, N_("_Insert") },
 		{ "MenuInsertNames",		NULL, N_("_Names") },
-		{ "MenuInsertObjects",		NULL, N_("_Objects") },
 		{ "MenuInsertSpecial",		NULL, N_("S_pecial") },
 	{ "MenuFormat",		NULL, N_("F_ormat") },
-		{ "MenuFormatFormat",		NULL, N_("_Format") },
 		{ "MenuFormatColumn",		NULL, N_("C_olumn") },
 		{ "MenuFormatRow",		NULL, N_("_Row") },
 		{ "MenuFormatSheet",		NULL, N_("_Sheet") },
@@ -1718,6 +1738,7 @@ static GtkActionEntry const permanent_actions[] = {
 		{ "MenuContingencyTests",	NULL,	N_("Contin_gency Table") },
 		{ "MenuToolForecast",	NULL,	N_("F_orecast") },
 		{ "MenuToolFrequencies",	NULL,	N_("Fre_quency Tables") },
+		{ "MenuToolTwoMedians",	NULL,	N_("Two _Medians") },
 		{ "MenuToolTTest",	NULL,	N_("Two _Means") },
 	{ "MenuData",		NULL, N_("_Data") },
 		{ "MenuFilter",		NULL,	N_("_Filter") },
@@ -1744,6 +1765,22 @@ static GtkActionEntry const permanent_actions[] = {
 		NULL, N_("Send the current file via email"),
 		G_CALLBACK (cb_file_sendto) },
 	{ "FilePrintArea",      NULL, N_("Print Area")},
+#ifdef HAVE_GTK_ADJUSTMENT_CONFIGURE
+	/* gtk_adjustment_configure implies gtk 2.14 or later */
+	/* that is required for GTK_STOCK_PAGE_SETUP */
+	{ "FilePageSetup", GTK_STOCK_PAGE_SETUP, N_("Page Set_up..."),
+#else
+	{ "FilePageSetup", NULL, N_("Page Set_up..."),
+#endif
+		NULL, N_("Setup the page settings for your current printer"),
+		G_CALLBACK (cb_file_page_setup) },
+	{ "FilePrintPreview", GTK_STOCK_PRINT_PREVIEW, NULL,
+		NULL, N_("Print preview"),
+		G_CALLBACK (cb_file_print_preview) },
+	{ "FilePrint", GTK_STOCK_PRINT, NULL,
+		"<control>p", N_("Print the current file"),
+		G_CALLBACK (cb_file_print) },
+
 	{ "FileHistoryFull", NULL, N_("Full _History..."),
 		NULL, N_("Access previously used file"),
 		G_CALLBACK (cb_file_history_full) },
@@ -1799,21 +1836,6 @@ static GtkActionEntry const permanent_actions[] = {
 
 static GtkActionEntry const actions[] = {
 /* File */
-#ifdef HAVE_GTK_ADJUSTMENT_CONFIGURE
-	/* gtk_adjustment_configure implies gtk 2.14 or later */
-	/* that is required for GTK_STOCK_PAGE_SETUP */
-	{ "FilePageSetup", GTK_STOCK_PAGE_SETUP, N_("Page Set_up..."),
-#else
-	{ "FilePageSetup", NULL, N_("Page Set_up..."),
-#endif
-		NULL, N_("Setup the page settings for your current printer"),
-		G_CALLBACK (cb_file_page_setup) },
-	{ "FilePrintPreview", GTK_STOCK_PRINT_PREVIEW, NULL,
-		NULL, N_("Print preview"),
-		G_CALLBACK (cb_file_print_preview) },
-	{ "FilePrint", GTK_STOCK_PRINT, NULL,
-		"<control>p", N_("Print the current file"),
-		G_CALLBACK (cb_file_print) },
 	{ "FileMetaData", GTK_STOCK_PROPERTIES, N_("Document Proper_ties..."),
 		NULL, N_("Edit document properties"),
 		G_CALLBACK (cb_doc_meta_data) },
@@ -2128,7 +2150,7 @@ static GtkActionEntry const actions[] = {
 		NULL, N_("Various frequency tables for numeric data"),
 		G_CALLBACK (cb_tools_histogram) },
 
-/* Tools -> Two Means */
+/* Tools -> Analysis -> Two Means */
 	{ "ToolTTestPaired", NULL, N_("_Paired Samples: T-Test..."),
 		NULL, N_("Comparing two population means for two paired samples: t-test..."),
 		G_CALLBACK (cb_tools_ttest_paired) },
@@ -2145,6 +2167,15 @@ static GtkActionEntry const actions[] = {
 		NULL, N_("Comparing two population means from populations with known variances: z-test..."),
 		G_CALLBACK (cb_tools_ztest) },
 
+/* Tools -> Analysis -> Two Medias */
+
+	{ "ToolsTwoMedianSignTest", NULL, N_("_Sign Test..."),
+		NULL, N_("Comparing the values of two medians of paired observations"),
+		G_CALLBACK (cb_tools_sign_test_two_medians) },
+	{ "ToolsTwoMedianWilcoxonMannWhitney", NULL, N_("_Wilcoxon-Mann-Whitney..."),
+		NULL, N_("Comparing the values of two medians of unpaired observations"),
+		G_CALLBACK (cb_tools_wilcoxon_mann_whitney) },
+
 /* Tools -> Analysis */
 	{ "ToolsCorrelation", NULL, N_("_Correlation..."),
 		NULL, N_("Pearson Correlation"),
@@ -2183,9 +2214,6 @@ static GtkActionEntry const actions[] = {
 	{ "ToolsOneMedianSignTest", NULL, N_("One M_edian Sign Test..."),
 		NULL, N_("Testing the value of a median"),
 		G_CALLBACK (cb_tools_sign_test_one_median) },
-	{ "ToolsTwoMedianSignTest", NULL, N_("T_wo Medians Sign Test..."),
-		NULL, N_("Comparing the values of two medians"),
-		G_CALLBACK (cb_tools_sign_test_two_medians) },
 /* Data */
 	{ "DataSort", GTK_STOCK_SORT_ASCENDING, N_("_Sort..."),
 		NULL, N_("Sort the selected region"),
@@ -2249,8 +2277,8 @@ static GtkActionEntry const actions[] = {
 	{ "DataAutoFilter", "Gnumeric_AutoFilter", N_("Add _Auto Filter"),
 		NULL, N_("Add or remove a filter"),
 		G_CALLBACK (cb_auto_filter) },
-	{ "DataFilterShowAll", NULL, N_("_Show All"),
-		NULL, N_("Show all filtered and hidden rows"),
+	{ "DataFilterShowAll", NULL, N_("_Clear Advanced Filter"),
+		NULL, N_("Show all rows hidden by an advanced filter"),
 		G_CALLBACK (cb_show_all) },
 	{ "DataFilterAdvancedfilter", NULL, N_("Advanced _Filter..."),
 		NULL, N_("Filter data with given criteria"),
