@@ -31,6 +31,7 @@
 #include <gnm-format.h>
 #include <number-match.h>
 #include <gnm-datetime.h>
+#include <gnumeric-gconf.h>
 #include <goffice/goffice.h>
 
 #include <gsf/gsf-impl-utils.h>
@@ -38,6 +39,10 @@
 #include <gdk/gdkkeysyms.h>
 #include <string.h>
 
+#ifndef HAVE_GTK_WIDGET_IS_TOPLEVEL
+#define gtk_widget_is_toplevel(w_) (GTK_WIDGET_FLAGS ((w_)) & GTK_TOPLEVEL)
+#endif
+
 typedef struct {
 	GnmRangeRef ref;
 	int	    text_start;
@@ -186,7 +191,7 @@ cb_icon_clicked (GtkButton *icon,
 	 * not rolled up.
 	 */
 
-	if (toplevel != NULL && (GTK_WIDGET_FLAGS (toplevel) & GTK_TOPLEVEL)) {
+	if (toplevel != NULL && gtk_widget_is_toplevel (toplevel)) {
 		GtkWidget *old_entry_parent;
 		GtkWidget *old_toplevel_child;
 		GParamSpec **container_props_pspec;
@@ -548,6 +553,7 @@ cb_gee_key_press_event (GtkEntry	*entry,
 			GnmExprEntry	*gee)
 {
 	WBCGtk *wbcg  = gee->wbcg;
+	gboolean is_enter = FALSE;
 	int state = gnumeric_filter_modifiers (event->state);
 
 	switch (event->keyval) {
@@ -603,10 +609,7 @@ cb_gee_key_press_event (GtkEntry	*entry,
 		return TRUE;
 
 	case GDK_KP_Enter:
-	case GDK_Return: {
-		SheetView *sv;
-		WBCEditResult result;
-
+	case GDK_Return:
 		if (gee->is_cell_renderer)
 			return FALSE;
 		/* Is this the right way to append a newline ?? */
@@ -621,19 +624,49 @@ cb_gee_key_press_event (GtkEntry	*entry,
 		if (!wbcg_is_editing (wbcg))
 			break;
 
-		/* Be careful to use the editing sheet */
-		sv = sheet_get_view (wbcg->editing_sheet,
-			wb_control_view (WORKBOOK_CONTROL (wbcg)));
+		is_enter = TRUE;
+		/* fall through */
 
-		if (state & GDK_CONTROL_MASK)
+	case GDK_Tab:
+	case GDK_ISO_Left_Tab:
+	case GDK_KP_Tab:
+		/* Tab is only applicable for the main entry */
+		 if (gee->is_cell_renderer || !wbcg_is_editing (wbcg))
+			break;
+		{
+		SheetView *sv;
+		WBCEditResult result;
+
+		if (is_enter && (state & GDK_CONTROL_MASK))
 			result = (state & GDK_SHIFT_MASK) ? WBC_EDIT_ACCEPT_ARRAY : WBC_EDIT_ACCEPT_RANGE;
 		else
 			result = WBC_EDIT_ACCEPT;
+
+		/* Be careful to restore the editing sheet if we are editing */
+		sv = sheet_get_view (wbcg->editing_sheet,
+			wb_control_view (WORKBOOK_CONTROL (wbcg)));
+
 		/* move the edit pos for normal entry */
 		if (wbcg_edit_finish (wbcg, result, NULL) && result == WBC_EDIT_ACCEPT) {
-			gboolean const direction = (event->state & GDK_SHIFT_MASK) ? FALSE : TRUE;
-			sv_selection_walk_step (sv, direction, FALSE);
-			sv_update (sv);
+			GODirection dir = gnm_conf_get_core_gui_editing_enter_moves_dir ();
+			if (!is_enter || dir != GO_DIRECTION_NONE) {
+				gboolean forward = TRUE;
+				gboolean horizontal = TRUE;
+				if (is_enter) {
+					horizontal = go_direction_is_horizontal (dir);
+					forward = go_direction_is_forward (dir);
+				}
+
+				if (event->state & GDK_SHIFT_MASK)
+					forward = !forward;
+
+				sv_selection_walk_step (sv, forward, horizontal);
+
+				/* invalidate, in case Enter direction changes */
+				if (is_enter)
+					sv->first_tab_col = -1;
+				sv_update (sv);
+			}
 		}
 		return TRUE;
 	}
@@ -828,7 +861,7 @@ gee_data_editor_set_format (GogDataEditor *deditor, GOFormat const *fmt)
 		g_free (txt);
 	}
 
-	if (v) value_release (v);
+	value_release (v);
 }
 
 static void
@@ -1045,6 +1078,12 @@ gee_rangesel_update_text (GnmExprEntry *gee)
 	if (text != NULL) {
 		/* Set the cursor at the end.  It looks nicer */
 		len = strlen (text);
+
+#ifdef HAVE_GTK_ENTRY_GET_BUFFER
+		/* We need to work around gtk bug #601922 */
+		gtk_entry_get_buffer (GTK_ENTRY (gee->entry));
+#endif
+
 		gtk_editable_insert_text (editable, text, len, &rs->text_end);
 		gtk_editable_set_position (editable, rs->text_end);
 		g_free (text);
@@ -1747,8 +1786,7 @@ gnm_expr_entry_get_text	(GnmExprEntry const *gee)
  * gnm_expr_entry_parse_as_value :
  *
  * @gee: GnmExprEntry
- * @sheet: the sheet where the cell range is evaluated. This really only needed if
- *         the range given does not include a sheet specification.
+ * @sheet: the sheet where the cell range is evaluated.
  *
  * Returns a (GnmValue *) of type VALUE_CELLRANGE if the @range was
  *	succesfully parsed or NULL on failure.
@@ -1756,10 +1794,32 @@ gnm_expr_entry_get_text	(GnmExprEntry const *gee)
 GnmValue *
 gnm_expr_entry_parse_as_value (GnmExprEntry *gee, Sheet *sheet)
 {
+	GnmParsePos pp;
+	GnmExprParseFlags flags = GNM_EXPR_PARSE_UNKNOWN_NAMES_ARE_STRINGS;
+	GnmValue *v;
+	const char *txt;
+
 	g_return_val_if_fail (IS_GNM_EXPR_ENTRY (gee), NULL);
 
-	return value_new_cellrange_str (sheet,
-		gtk_entry_get_text (gnm_expr_entry_get_entry (gee)));
+	if ((gee->flags & GNM_EE_FORCE_ABS_REF))
+		flags |= GNM_EXPR_PARSE_FORCE_ABSOLUTE_REFERENCES;
+	else if ((gee->flags & GNM_EE_FORCE_REL_REF))
+		flags |= GNM_EXPR_PARSE_FORCE_RELATIVE_REFERENCES;
+	if (!(gee->flags & GNM_EE_SHEET_OPTIONAL))
+		flags |= GNM_EXPR_PARSE_FORCE_EXPLICIT_SHEET_REFERENCES;
+
+	txt = gtk_entry_get_text (gnm_expr_entry_get_entry (gee));
+
+	parse_pos_init_sheet (&pp, sheet);
+	v = value_new_cellrange_parsepos_str (&pp, txt, flags);
+
+	if (!v && (gee->flags & GNM_EE_CONSTANT_ALLOWED)) {
+		GODateConventions const *date_conv =
+			sheet ? workbook_date_conv (sheet->workbook) : NULL;
+		v = format_match_number (txt, NULL, date_conv);
+	}
+
+	return v;
 }
 
 /**
